@@ -1,30 +1,18 @@
 import copy
 import sys
-
-from datetime import datetime
 from contextlib import contextmanager
 
 from sentry_sdk._compat import with_metaclass
 from sentry_sdk.consts import INSTRUMENTER
 from sentry_sdk.scope import Scope
 from sentry_sdk.client import Client
-from sentry_sdk.profiler import Profile
 from sentry_sdk.tracing import (
     NoOpSpan,
     Span,
     Transaction,
-    BAGGAGE_HEADER_NAME,
-    SENTRY_TRACE_HEADER_NAME,
-)
-from sentry_sdk.session import Session
-from sentry_sdk.tracing_utils import (
-    has_tracing_enabled,
-    normalize_incoming_data,
 )
 
 from sentry_sdk.utils import (
-    exc_info_from_error,
-    event_from_exception,
     logger,
     ContextVar,
 )
@@ -32,18 +20,18 @@ from sentry_sdk.utils import (
 from sentry_sdk._types import TYPE_CHECKING
 
 if TYPE_CHECKING:
-    from typing import Union
     from typing import Any
-    from typing import Optional
-    from typing import Tuple
-    from typing import Dict
-    from typing import List
     from typing import Callable
+    from typing import ContextManager
+    from typing import Dict
     from typing import Generator
+    from typing import List
+    from typing import Optional
+    from typing import overload
+    from typing import Tuple
     from typing import Type
     from typing import TypeVar
-    from typing import overload
-    from typing import ContextManager
+    from typing import Union
 
     from sentry_sdk.integrations import Integration
     from sentry_sdk._types import (
@@ -67,24 +55,6 @@ else:
 _local = ContextVar("sentry_current_hub")
 
 
-def _update_scope(base, scope_change, scope_kwargs):
-    # type: (Scope, Optional[Any], Dict[str, Any]) -> Scope
-    if scope_change and scope_kwargs:
-        raise TypeError("cannot provide scope and kwargs")
-    if scope_change is not None:
-        final_scope = copy.copy(base)
-        if callable(scope_change):
-            scope_change(final_scope)
-        else:
-            final_scope.update_from_scope(scope_change)
-    elif scope_kwargs:
-        final_scope = copy.copy(base)
-        final_scope.update_from_kwargs(**scope_kwargs)
-    else:
-        final_scope = base
-    return final_scope
-
-
 def _should_send_default_pii():
     # type: () -> bool
     client = Hub.current.client
@@ -295,18 +265,9 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         If the return value is not `None` the hub is guaranteed to have a
         client attached.
         """
-        if isinstance(name_or_class, str):
-            integration_name = name_or_class
-        elif name_or_class.identifier is not None:
-            integration_name = name_or_class.identifier
-        else:
-            raise ValueError("Integration has no name")
-
         client = self.client
         if client is not None:
-            rv = client.integrations.get(integration_name)
-            if rv is not None:
-                return rv
+            return client.get_integration(name_or_class)
 
     @property
     def client(self):
@@ -322,7 +283,15 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
 
     def last_event_id(self):
         # type: () -> Optional[str]
-        """Returns the last event ID."""
+        """
+        Returns the last event ID.
+
+        .. deprecated:: 1.40.5
+            This function is deprecated and will be removed in a future release. The functions `capture_event`, `capture_message`, and `capture_exception` return the event ID directly.
+        """
+        logger.warning(
+            "Deprecated: last_event_id is deprecated. This will be removed in the future. The functions `capture_event`, `capture_message`, and `capture_exception` return the event ID directly."
+        )
         return self._last_event_id
 
     def bind_client(
@@ -333,76 +302,100 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         top = self._stack[-1]
         self._stack[-1] = (new, top[1])
 
-    def capture_event(self, event, hint=None, scope=None, **scope_args):
+    def capture_event(self, event, hint=None, scope=None, **scope_kwargs):
         # type: (Event, Optional[Hint], Optional[Scope], Any) -> Optional[str]
         """
         Captures an event.
 
-        Alias of :py:meth:`sentry_sdk.Client.capture_event`.
+        Alias of :py:meth:`sentry_sdk.Scope.capture_event`.
 
-        :param scope_args: For supported `**scope_args` see
-            :py:meth:`sentry_sdk.Scope.update_from_kwargs`.
+        :param event: A ready-made event that can be directly sent to Sentry.
+
+        :param hint: Contains metadata about the event that can be read from `before_send`, such as the original exception object or a HTTP request object.
+
+        :param scope: An optional :py:class:`sentry_sdk.Scope` to apply to events.
+            The `scope` and `scope_kwargs` parameters are mutually exclusive.
+
+        :param scope_kwargs: Optional data to apply to event.
+            For supported `**scope_kwargs` see :py:meth:`sentry_sdk.Scope.update_from_kwargs`.
+            The `scope` and `scope_kwargs` parameters are mutually exclusive.
         """
         client, top_scope = self._stack[-1]
-        scope = _update_scope(top_scope, scope, scope_args)
-        if client is not None:
-            is_transaction = event.get("type") == "transaction"
-            rv = client.capture_event(event, hint, scope)
-            if rv is not None and not is_transaction:
-                self._last_event_id = rv
-            return rv
-        return None
+        if client is None:
+            return None
 
-    def capture_message(self, message, level=None, scope=None, **scope_args):
+        last_event_id = top_scope.capture_event(
+            event, hint, client=client, scope=scope, **scope_kwargs
+        )
+
+        is_transaction = event.get("type") == "transaction"
+        if last_event_id is not None and not is_transaction:
+            self._last_event_id = last_event_id
+
+        return last_event_id
+
+    def capture_message(self, message, level=None, scope=None, **scope_kwargs):
         # type: (str, Optional[str], Optional[Scope], Any) -> Optional[str]
         """
         Captures a message.
 
-        :param message: The string to send as the message.
+        Alias of :py:meth:`sentry_sdk.Scope.capture_message`.
+
+        :param message: The string to send as the message to Sentry.
 
         :param level: If no level is provided, the default level is `info`.
 
-        :param scope: An optional :py:class:`sentry_sdk.Scope` to use.
+        :param scope: An optional :py:class:`sentry_sdk.Scope` to apply to events.
+            The `scope` and `scope_kwargs` parameters are mutually exclusive.
 
-        :param scope_args: For supported `**scope_args` see
-            :py:meth:`sentry_sdk.Scope.update_from_kwargs`.
+        :param scope_kwargs: Optional data to apply to event.
+            For supported `**scope_kwargs` see :py:meth:`sentry_sdk.Scope.update_from_kwargs`.
+            The `scope` and `scope_kwargs` parameters are mutually exclusive.
 
         :returns: An `event_id` if the SDK decided to send the event (see :py:meth:`sentry_sdk.Client.capture_event`).
         """
-        if self.client is None:
+        client, top_scope = self._stack[-1]
+        if client is None:
             return None
-        if level is None:
-            level = "info"
-        return self.capture_event(
-            {"message": message, "level": level}, scope=scope, **scope_args
+
+        last_event_id = top_scope.capture_message(
+            message, level=level, client=client, scope=scope, **scope_kwargs
         )
 
-    def capture_exception(self, error=None, scope=None, **scope_args):
+        if last_event_id is not None:
+            self._last_event_id = last_event_id
+
+        return last_event_id
+
+    def capture_exception(self, error=None, scope=None, **scope_kwargs):
         # type: (Optional[Union[BaseException, ExcInfo]], Optional[Scope], Any) -> Optional[str]
         """Captures an exception.
 
-        :param error: An exception to catch. If `None`, `sys.exc_info()` will be used.
+        Alias of :py:meth:`sentry_sdk.Scope.capture_exception`.
 
-        :param scope_args: For supported `**scope_args` see
-            :py:meth:`sentry_sdk.Scope.update_from_kwargs`.
+        :param error: An exception to capture. If `None`, `sys.exc_info()` will be used.
+
+        :param scope: An optional :py:class:`sentry_sdk.Scope` to apply to events.
+            The `scope` and `scope_kwargs` parameters are mutually exclusive.
+
+        :param scope_kwargs: Optional data to apply to event.
+            For supported `**scope_kwargs` see :py:meth:`sentry_sdk.Scope.update_from_kwargs`.
+            The `scope` and `scope_kwargs` parameters are mutually exclusive.
 
         :returns: An `event_id` if the SDK decided to send the event (see :py:meth:`sentry_sdk.Client.capture_event`).
         """
-        client = self.client
+        client, top_scope = self._stack[-1]
         if client is None:
             return None
-        if error is not None:
-            exc_info = exc_info_from_error(error)
-        else:
-            exc_info = sys.exc_info()
 
-        event, hint = event_from_exception(exc_info, client_options=client.options)
-        try:
-            return self.capture_event(event, hint=hint, scope=scope, **scope_args)
-        except Exception:
-            self._capture_internal_exception(sys.exc_info())
+        last_event_id = top_scope.capture_exception(
+            error, client=client, scope=scope, **scope_kwargs
+        )
 
-        return None
+        if last_event_id is not None:
+            self._last_event_id = last_event_id
+
+        return last_event_id
 
     def _capture_internal_exception(
         self, exc_info  # type: Any
@@ -412,6 +405,8 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         Capture an exception that is likely caused by a bug in the SDK
         itself.
 
+        Duplicated in :py:meth:`sentry_sdk.Client._capture_internal_exception`.
+
         These exceptions do not end up in Sentry and are just logged instead.
         """
         logger.error("Internal error in sentry_sdk", exc_info=exc_info)
@@ -431,31 +426,9 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
             logger.info("Dropped breadcrumb because no client bound")
             return
 
-        crumb = dict(crumb or ())  # type: Breadcrumb
-        crumb.update(kwargs)
-        if not crumb:
-            return
+        kwargs["client"] = client
 
-        hint = dict(hint or ())  # type: Hint
-
-        if crumb.get("timestamp") is None:
-            crumb["timestamp"] = datetime.utcnow()
-        if crumb.get("type") is None:
-            crumb["type"] = "default"
-
-        if client.options["before_breadcrumb"] is not None:
-            new_crumb = client.options["before_breadcrumb"](crumb, hint)
-        else:
-            new_crumb = crumb
-
-        if new_crumb is not None:
-            scope._breadcrumbs.append(new_crumb)
-        else:
-            logger.info("before breadcrumb dropped breadcrumb (%s)", crumb)
-
-        max_breadcrumbs = client.options["max_breadcrumbs"]  # type: int
-        while len(scope._breadcrumbs) > max_breadcrumbs:
-            scope._breadcrumbs.popleft()
+        scope.add_breadcrumb(crumb, hint, **kwargs)
 
     def start_span(self, span=None, instrumenter=INSTRUMENTER.SENTRY, **kwargs):
         # type: (Optional[Span], str, Any) -> Span
@@ -474,54 +447,12 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
 
         For supported `**kwargs` see :py:class:`sentry_sdk.tracing.Span`.
         """
-        configuration_instrumenter = self.client and self.client.options["instrumenter"]
+        client, scope = self._stack[-1]
 
-        if instrumenter != configuration_instrumenter:
-            return NoOpSpan()
+        kwargs["hub"] = self
+        kwargs["client"] = client
 
-        # THIS BLOCK IS DEPRECATED
-        # TODO: consider removing this in a future release.
-        # This is for backwards compatibility with releases before
-        # start_transaction existed, to allow for a smoother transition.
-        if isinstance(span, Transaction) or "transaction" in kwargs:
-            deprecation_msg = (
-                "Deprecated: use start_transaction to start transactions and "
-                "Transaction.start_child to start spans."
-            )
-
-            if isinstance(span, Transaction):
-                logger.warning(deprecation_msg)
-                return self.start_transaction(span)
-
-            if "transaction" in kwargs:
-                logger.warning(deprecation_msg)
-                name = kwargs.pop("transaction")
-                return self.start_transaction(name=name, **kwargs)
-
-        # THIS BLOCK IS DEPRECATED
-        # We do not pass a span into start_span in our code base, so I deprecate this.
-        if span is not None:
-            deprecation_msg = "Deprecated: passing a span into `start_span` is deprecated and will be removed in the future."
-            logger.warning(deprecation_msg)
-            return span
-
-        kwargs.setdefault("hub", self)
-
-        active_span = self.scope.span
-        if active_span is not None:
-            new_child_span = active_span.start_child(**kwargs)
-            return new_child_span
-
-        # If there is already a trace_id in the propagation context, use it.
-        # This does not need to be done for `start_child` above because it takes
-        # the trace_id from the parent span.
-        if "trace_id" not in kwargs:
-            traceparent = self.get_traceparent()
-            trace_id = traceparent.split("-")[0] if traceparent else None
-            if trace_id is not None:
-                kwargs["trace_id"] = trace_id
-
-        return Span(**kwargs)
+        return scope.start_span(span=span, instrumenter=instrumenter, **kwargs)
 
     def start_transaction(
         self, transaction=None, instrumenter=INSTRUMENTER.SENTRY, **kwargs
@@ -551,55 +482,25 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
 
         For supported `**kwargs` see :py:class:`sentry_sdk.tracing.Transaction`.
         """
-        configuration_instrumenter = self.client and self.client.options["instrumenter"]
+        client, scope = self._stack[-1]
 
-        if instrumenter != configuration_instrumenter:
-            return NoOpSpan()
+        kwargs["hub"] = self
+        kwargs["client"] = client
 
-        custom_sampling_context = kwargs.pop("custom_sampling_context", {})
-
-        # if we haven't been given a transaction, make one
-        if transaction is None:
-            kwargs.setdefault("hub", self)
-            transaction = Transaction(**kwargs)
-
-        # use traces_sample_rate, traces_sampler, and/or inheritance to make a
-        # sampling decision
-        sampling_context = {
-            "transaction_context": transaction.to_json(),
-            "parent_sampled": transaction.parent_sampled,
-        }
-        sampling_context.update(custom_sampling_context)
-        transaction._set_initial_sampling_decision(sampling_context=sampling_context)
-
-        profile = Profile(transaction, hub=self)
-        profile._set_initial_sampling_decision(sampling_context=sampling_context)
-
-        # we don't bother to keep spans if we already know we're not going to
-        # send the transaction
-        if transaction.sampled:
-            max_spans = (
-                self.client and self.client.options["_experiments"].get("max_spans")
-            ) or 1000
-            transaction.init_span_recorder(maxlen=max_spans)
-
-        return transaction
+        return scope.start_transaction(
+            transaction=transaction, instrumenter=instrumenter, **kwargs
+        )
 
     def continue_trace(self, environ_or_headers, op=None, name=None, source=None):
         # type: (Dict[str, Any], Optional[str], Optional[str], Optional[str]) -> Transaction
         """
         Sets the propagation context from environment or headers and returns a transaction.
         """
-        with self.configure_scope() as scope:
-            scope.generate_propagation_context(environ_or_headers)
+        scope = self._stack[-1][1]
 
-        transaction = Transaction.continue_from_headers(
-            normalize_incoming_data(environ_or_headers),
-            op=op,
-            name=name,
-            source=source,
+        return scope.continue_trace(
+            environ_or_headers=environ_or_headers, op=op, name=name, source=source
         )
-        return transaction
 
     @overload
     def push_scope(
@@ -713,12 +614,9 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
     ):
         # type: (...) -> None
         """Starts a new session."""
-        self.end_session()
         client, scope = self._stack[-1]
-        scope._session = Session(
-            release=client.options["release"] if client else None,
-            environment=client.options["environment"] if client else None,
-            user=scope._user,
+        scope.start_session(
+            client=client,
             session_mode=session_mode,
         )
 
@@ -726,13 +624,7 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         # type: (...) -> None
         """Ends the current session if there is one."""
         client, scope = self._stack[-1]
-        session = scope._session
-        self.scope._session = None
-
-        if session is not None:
-            session.close()
-            if client is not None:
-                client.capture_session(session)
+        scope.end_session(client=client)
 
     def stop_auto_session_tracking(self):
         # type: (...) -> None
@@ -741,9 +633,8 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         This temporarily session tracking for the current scope when called.
         To resume session tracking call `resume_auto_session_tracking`.
         """
-        self.end_session()
         client, scope = self._stack[-1]
-        scope._force_auto_session_tracking = False
+        scope.stop_auto_session_tracking(client=client)
 
     def resume_auto_session_tracking(self):
         # type: (...) -> None
@@ -751,8 +642,8 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         disabled earlier.  This requires that generally automatic session
         tracking is enabled.
         """
-        client, scope = self._stack[-1]
-        scope._force_auto_session_tracking = None
+        scope = self._stack[-1][1]
+        scope.resume_auto_session_tracking()
 
     def flush(
         self,
@@ -772,25 +663,16 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         """
         Returns the traceparent either from the active span or from the scope.
         """
-        if self.client is not None:
-            if has_tracing_enabled(self.client.options) and self.scope.span is not None:
-                return self.scope.span.to_traceparent()
-
-        return self.scope.get_traceparent()
+        client, scope = self._stack[-1]
+        return scope.get_traceparent(client=client)
 
     def get_baggage(self):
         # type: () -> Optional[str]
         """
         Returns Baggage either from the active span or from the scope.
         """
-        if (
-            self.client is not None
-            and has_tracing_enabled(self.client.options)
-            and self.scope.span is not None
-        ):
-            baggage = self.scope.span.to_baggage()
-        else:
-            baggage = self.scope.get_baggage()
+        client, scope = self._stack[-1]
+        baggage = scope.get_baggage(client=client)
 
         if baggage is not None:
             return baggage.serialize()
@@ -804,19 +686,9 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         from the span representing the request, if available, or the current
         span on the scope if not.
         """
-        client = self._stack[-1][0]
-        propagate_traces = client and client.options["propagate_traces"]
-        if not propagate_traces:
-            return
+        client, scope = self._stack[-1]
 
-        span = span or self.scope.span
-
-        if client and has_tracing_enabled(client.options) and span is not None:
-            for header in span.iter_headers():
-                yield header
-        else:
-            for header in self.scope.iter_headers():
-                yield header
+        return scope.iter_trace_propagation_headers(span=span, client=client)
 
     def trace_propagation_meta(self, span=None):
         # type: (Optional[Span]) -> str
@@ -829,23 +701,8 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
                 "The parameter `span` in trace_propagation_meta() is deprecated and will be removed in the future."
             )
 
-        meta = ""
-
-        sentry_trace = self.get_traceparent()
-        if sentry_trace is not None:
-            meta += '<meta name="%s" content="%s">' % (
-                SENTRY_TRACE_HEADER_NAME,
-                sentry_trace,
-            )
-
-        baggage = self.get_baggage()
-        if baggage is not None:
-            meta += '<meta name="%s" content="%s">' % (
-                BAGGAGE_HEADER_NAME,
-                baggage,
-            )
-
-        return meta
+        client, scope = self._stack[-1]
+        return scope.trace_propagation_meta(span=span, client=client)
 
 
 GLOBAL_HUB = Hub()
